##############################################################################
#
# BernArticle - yet another Plone article...
#
# Copyright (C) 2005 4teamwork Ltd., Bern, Switzerland and
#                    Informatikdienste of the city of Bern, Switzerland
#
# GNU General Public Licence (GPL)
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307 USA
#
############################################################################## 

from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.migrations.migration_util import safeEditProperty


#***********************************************************************************
# helper methods

def safeAppendStringToTuple(self, newString, tupleOfStrings):
    """helper function to appends a string to a tuple and make sure it is in there only once"""
    listOfStrings = list(tupleOfStrings)
    if newString not in listOfStrings:
        listOfStrings.append(newString)
    return tuple(listOfStrings)

def safeAppendToLinesProperty(self, newLine, properties, propertyName):
    lines = properties.getProperty(propertyName)
    lines = safeAppendStringToTuple(self, newLine, lines)
    safeEditProperty(properties, key=propertyName, data_type='lines', value=lines)

def safeRemoveLineFromProperty(self, lineToDelete, properties, propertyName):  
    lines = list(properties.getProperty(propertyName))
    try:
        lines.remove(lineToDelete)
        safeEditProperty(properties, key=propertyName, data_type='lines', value=tuple(lines))
    except:
        # line was not in values - remove was not possible and is not needed
        pass


#***********************************************************************************
# reset registry to initial (empty) state

def resetBernArticleRegistrationProperties(self):
    """initializes type registry to a empty one"""
    portal_properties = getToolByName(self, 'portal_properties')
    bernArticle_properties = getattr(portal_properties, 'BernArticle_properties', None)
    safeEditProperty(bernArticle_properties, key='BernArticle_types_for_search', data_type='lines', value=tuple())
    safeEditProperty(bernArticle_properties, key='BernArticle_types_for_collector', data_type='lines', value=tuple())
    safeEditProperty(bernArticle_properties, key='BernArticle_types_for_article_folder_contents', data_type='lines', value=tuple())
    safeEditProperty(bernArticle_properties, key='BernArticle_article_types', data_type='lines', value=tuple())
    safeEditProperty(bernArticle_properties, key='BernArticle_folder_types', data_type='lines', value=tuple())
    safeEditProperty(bernArticle_properties, key='BernArticle_teaser_block_types', data_type='lines', value=tuple())
    safeEditProperty(bernArticle_properties, key='BernArticle_block_types', data_type='lines', value=tuple())
    safeEditProperty(bernArticle_properties, key='BernArticle_collector_types', data_type='lines', value=tuple())
    

#***********************************************************************************
# register a new type

def registerBernArticleType(self, newTypeName, isArticleType=False, isCollectorType=False, isFolderType=False, isTeaserType=False, isCalendarType=False, isHiddenFromNav=True, isInSearchResults=True,
                            isInCollectorResultsAndBlockLists=False, isInFolderContents=True, isLinkableKupuType=True):
    """registers a new type with BernArticle"""
    
    # is it a calendar type? yes: register it with calender tool
    if isCalendarType:
        portal_calendar = getToolByName(self, 'portal_calendar')
        ctypes = list(portal_calendar.getCalendarTypes())
        use_session = portal_calendar.getUseSession()
        if newTypeName not in ctypes:
            ctypes.append(newTypeName)
        portal_calendar.edit_configuration(tuple(ctypes), use_session)

    # should type be hidden from nav? yes: exclude it in navigation properties
    if isHiddenFromNav:
        portal_properties = getToolByName(self, 'portal_properties')
        navtree_properties = getattr(portal_properties, 'navtree_properties', None)
        safeAppendToLinesProperty(self, newTypeName, navtree_properties, 'metaTypesNotToList')

    # append the new type to the relevant properties
    portal_properties = getToolByName(self, 'portal_properties')
    bernArticle_properties = getattr(portal_properties, 'BernArticle_properties', None)

    if isArticleType:
        safeAppendToLinesProperty(self, newTypeName, bernArticle_properties, 'BernArticle_article_types')

    if isCollectorType:
        safeAppendToLinesProperty(self, newTypeName, bernArticle_properties, 'BernArticle_collector_types')

    if not (isCollectorType or isArticleType):
        safeAppendToLinesProperty(self, newTypeName, bernArticle_properties, 'BernArticle_block_types')

    if isFolderType:
        safeAppendToLinesProperty(self, newTypeName, bernArticle_properties, 'BernArticle_folder_types')

    if isTeaserType:
        safeAppendToLinesProperty(self, newTypeName, bernArticle_properties, 'BernArticle_teaser_block_types')

    if isInSearchResults:
        safeAppendToLinesProperty(self, newTypeName, bernArticle_properties, 'BernArticle_types_for_search')

    if isInCollectorResultsAndBlockLists:
        safeAppendToLinesProperty(self, newTypeName, bernArticle_properties, 'BernArticle_types_for_collector')

        # add new block to BernArticleBlockList container FTI allowed types:
        portal_types = getToolByName(self, 'portal_types')
        fti = portal_types.getTypeInfo('BernArticleBlockList')
        allowed_content_types = fti.allowed_content_types
        allowed_content_types = safeAppendStringToTuple(self, newTypeName, allowed_content_types)
        fti.manage_changeProperties(filter_content_types=1, allowed_content_types=allowed_content_types)

        # add new block to BernArticleCollector container FTI allowed types:
        portal_types = getToolByName(self, 'portal_types')
        fti = portal_types.getTypeInfo('BernArticleCollector')
        allowed_content_types = fti.allowed_content_types
        allowed_content_types = safeAppendStringToTuple(self, newTypeName, allowed_content_types)
        fti.manage_changeProperties(filter_content_types=1, allowed_content_types=allowed_content_types)


    if isInFolderContents:
        safeAppendToLinesProperty(self, newTypeName, bernArticle_properties, 'BernArticle_types_for_article_folder_contents')
      
    # add new type to BernArticle container FTI allowed types:
    portal_types = getToolByName(self, 'portal_types')
    fti = portal_types.getTypeInfo('BernArticle')
    allowed_content_types = fti.allowed_content_types
    allowed_content_types = safeAppendStringToTuple(self, newTypeName, allowed_content_types)
    fti.manage_changeProperties(filter_content_types=1, allowed_content_types=allowed_content_types)

    # should new type be linkable inside editor kupu?
    if isLinkableKupuType:
        kupu_library_tool = getToolByName(self, 'kupu_library_tool')
        resource_types = kupu_library_tool._res_types
        linkable_types = resource_types['linkable']
        linkable_types = safeAppendStringToTuple(self, newTypeName, linkable_types)
        resource_types['linkable'] = linkable_types
        kupu_library_tool._res_types.update(resource_types)

    # should new type be a collection inside editor kupu?
    if isFolderType:
        kupu_library_tool = getToolByName(self, 'kupu_library_tool')
        resource_types = kupu_library_tool._res_types
        collection_types = resource_types['collection']
        collection_types = safeAppendStringToTuple(self, newTypeName, collection_types)
        resource_types['collection'] = collection_types
        kupu_library_tool._res_types.update(resource_types)
    
    # allow article and collector type to be default pages for a folder (thus they can be a dynamic view):    
    if isArticleType or isCollectorType:
        portal_properties = getToolByName(self, 'portal_properties')
        site_properties = getattr(portal_properties, 'site_properties', None)
        safeAppendToLinesProperty(self, newTypeName, site_properties, 'default_page_types') 
    
    # new type should be created by portal_factory:
    factory = getToolByName(self, 'portal_factory')
    factoryTypes = factory.getFactoryTypes().keys()
    factoryTypes.append(newTypeName)
    factory.manage_setPortalFactoryTypes(listOfTypeIds=factoryTypes) 


#***********************************************************************************
# unregister a type

def unregisterBernArticleType(self, typeName):
    """unregisters a type with BernArticle"""

    # is it a calendar type? yes: unregister it with calender tool
    portal_calendar = getToolByName(self, 'portal_calendar')
    ctypes = list(portal_calendar.getCalendarTypes())
    try:
        ctypes.remove(typeName)
        use_session = portal_calendar.getUseSession()
        portal_calendar.edit_configuration(tuple(ctypes), use_session)
    except:
        # was not a clendar type - remove was not possible and is not needed
        pass

    # was the type hidden from nav? yes: remove it from navigation properties
    portal_properties = getToolByName(self, 'portal_properties')
    navtree_properties = getattr(portal_properties, 'navtree_properties', None)
    safeRemoveLineFromProperty(self, typeName, navtree_properties, 'metaTypesNotToList')

    # remove the block type from relevant properties
    portal_properties = getToolByName(self, 'portal_properties')
    bernArticle_properties = getattr(portal_properties, 'BernArticle_properties', None)
    safeRemoveLineFromProperty(self, typeName, bernArticle_properties, 'BernArticle_types_for_search')
    safeRemoveLineFromProperty(self, typeName, bernArticle_properties, 'BernArticle_article_types')
    safeRemoveLineFromProperty(self, typeName, bernArticle_properties, 'BernArticle_collector_types')
    safeRemoveLineFromProperty(self, typeName, bernArticle_properties, 'BernArticle_block_types')
    safeRemoveLineFromProperty(self, typeName, bernArticle_properties, 'BernArticle_folder_types')
    safeRemoveLineFromProperty(self, typeName, bernArticle_properties, 'BernArticle_teaser_block_types')
    safeRemoveLineFromProperty(self, typeName, bernArticle_properties, 'BernArticle_types_for_collector')
    safeRemoveLineFromProperty(self, typeName, bernArticle_properties, 'BernArticle_types_for_article_folder_contents')

    # remove block type from BernArticle container FTI allowed types:
    portal_types = getToolByName(self, 'portal_types')
    fti = portal_types.getTypeInfo('BernArticle')
    allowed_content_types = list(fti.allowed_content_types)
    try:
        allowed_content_types.remove(typeName)
        fti.manage_changeProperties(filter_content_types=1, allowed_content_types=tuple(allowed_content_types))
    except:
        # was not in allowed content types - remove was not possible and is not needed
        pass

    # remove block type from BernArticleBlockList container FTI allowed types:
    portal_types = getToolByName(self, 'portal_types')
    fti = portal_types.getTypeInfo('BernArticleBlockList')
    allowed_content_types = list(fti.allowed_content_types)
    try:
        allowed_content_types.remove(typeName)
        fti.manage_changeProperties(filter_content_types=1, allowed_content_types=tuple(allowed_content_types))
    except:
        # was not in allowed content types - remove was not possible and is not needed
        pass

    # remove block type from BernArticleCollector container FTI allowed types:
    portal_types = getToolByName(self, 'portal_types')
    fti = portal_types.getTypeInfo('BernArticleCollector')
    allowed_content_types = list(fti.allowed_content_types)
    try:
        allowed_content_types.remove(typeName)
        fti.manage_changeProperties(filter_content_types=1, allowed_content_types=tuple(allowed_content_types))
    except:
        # was not in allowed content types - remove was not possible and is not needed
        pass

    # is the type linkable inside editor kupu?
    kupu_library_tool = getToolByName(self, 'kupu_library_tool')
    resource_types = kupu_library_tool._res_types
    linkable_types = list(resource_types['linkable'])
    try:
        linkable_types.remove(typeName)
        resource_types['linkable'] = tuple(linkable_types)
        kupu_library_tool._res_types.update(resource_types)
    except:
        # was not in linkable types - remove was not possible and is not needed
        pass

    # is the type a collection inside editor kupu?
    resource_types = kupu_library_tool._res_types
    collection_types = list(resource_types['collection'])
    try:
        collection_types.remove(typeName)
        resource_types['collection'] = tuple(collection_types)
        kupu_library_tool._res_types.update(resource_types)
    except:
        # was not in collection types - remove was not possible and is not needed
        pass

