##############################################################################
#
# BernArticle - yet another Plone article...
#
# Copyright (C) 2005 4teamwork Ltd., Bern, Switzerland and
#                    Informatikdienste of the city of Bern, Switzerland
#
# GNU General Public Licence (GPL)
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307 USA
#
############################################################################## 

from HTMLParser import HTMLParser,HTMLParseError,piclose, charref, entityref
from string import lower,find
import urlparse

class HTML2SafeHTML(HTMLParser):
    
    can_close   = ['li','p','dd','dt','option']
    never_close = ['br','wbr','hr','input','isindex','base','meta','img']
    attrs_class_accept = ['odd', 'even']
    
    def __init__(self,valid_tags):
        HTMLParser.__init__(self)
        self.valid_tags = valid_tags
        self.result = ""
        self.openTags = []
        
    def end_tag(self,tag):
        self.result = "%s</%s>" % (self.result, tag)
            
    def handle_data(self, data):
        if data:
            self.result = self.result + data

    def handle_charref(self, name):
        self.result = "%s&#%s;" % (self.result, name)
        
    from htmlentitydefs import entitydefs # our entity defs list to use
    
    def handle_entityref(self, name):
        # this quotes non-standard entities
        if self.entitydefs.has_key(name): 
            amp = '&'
        else:
            amp = '&amp;'
        self.result = "%s%s%s;" % (self.result, amp, name)

    def handle_starttag(self, tag, attrs):
        """ Delete all tags except for legal ones """

        if tag in self.valid_tags:
            
            self.result = self.result + '<' + tag
            
            for k, v in attrs:
                if (tag=="a" and k=="href") or (tag=="img" and k=="src"):
                    v = self.getRelativeUrl(self.pageurl, v)
            
                if v is None:
                    self.result += ' ' + k
                else:
                    if lower(k[0:2]) != 'on' and lower(v[0:10]) != 'javascript' and lower(k[0:4]) != 'lang' and lower(k[0:6]) != 'border' and lower(k[0:5]) != 'style':
                        if lower(k[0:5]) != 'class' or lower(v) in self.attrs_class_accept:
                            self.result += ' %s="%s"' % (k, v)
                    
            if tag not in self.never_close:
                self.openTags.append(tag)
                
            self.result = self.result + '>'
                
    def handle_endtag(self, tag):

        try:            

            while tag != self.openTags[-1] and self.openTags[-1] in self.can_close:
                self.openTags.pop()
            
            if tag==self.openTags[-1]:
                self.end_tag(self.openTags.pop())                
                
        except IndexError:
            pass

    def cleanup(self):
        """ Append missing closing tags """
        while self.openTags:
            tag = self.openTags.pop()
            if tag not in self.can_close:
                self.end_tag(tag)

    def parse_starttag(self,i):
        try:
            return HTMLParser.parse_starttag(self,i)
        except HTMLParseError:
            try:
                return piclose.search(self.rawdata,i).end()
            except AttributeError:
                return -1
        
    def parse_endtag(self,i):
        try:
            return HTMLParser.parse_endtag(self,i)
        except HTMLParseError:
            try:
                return piclose.search(self.rawdata,i).end()
            except:
                return -1

    def goahead(self,end):

        # fix incomplete entity and char refs        
        rawdata = self.rawdata
        
        i = 0
        n = len(rawdata)
        newdata=''
        
        while i < n:
            j = find(rawdata,'&',i)
            if j==-1:
                break
            newdata = newdata + rawdata[i:j]
            if charref.match(rawdata, j) or entityref.match(rawdata, j):
                newdata = newdata + '&'
            else:
                newdata = newdata + '&amp;'
            i = j+1
            
        self.rawdata = newdata + rawdata[i:]

        # do normal parsing
        try:
            return HTMLParser.goahead(self,end)
        except HTMLParseError:
            pass
        
    def getRelativeUrl(self, pageUrl, targetUrl):
        """ Returns relative path from targetUrl to pageUrl. """
    
        # Just some shortcuts...
        SCHEME, HOSTNAME, PATH, PARAMS, QUERY, FRAGMENT = range(6)
    
        pageUnits = urlparse.urlparse(pageUrl)
        targetUnits = urlparse.urlparse(targetUrl)
    
        if pageUnits[:PATH] != targetUnits[:PATH]:
            # Scheme (http) or host:port (www.comain.com) are different
            # -> no possible relative URL
            return targetUrl
    
        afterPath = targetUnits[PARAMS:]
    
        pagePath = pageUnits[PATH].split('/')
        pagePath.reverse()
    
        targetPath = targetUnits[PATH].split('/')
        targetPath.reverse()
    
        # Remove parts which are equal (['a', 'b'] ['a', 'c'] --> ['c'])
        while (len(pagePath) > 0
           and len(targetPath) > 0
           and pagePath[-1] == targetPath[-1]):
            pagePath.pop()
            last_pop = targetPath.pop()
        if len(pagePath) == 0 and len(targetPath) == 0:
            # Special case: link to itself (http://foo/a http://foo/a -> a)
            targetPath.append(last_pop)
        if len(pagePath) == 1 and pagePath[0] == "" and len(targetPath) == 0:
            # Special case: (http://foo/a/ http://foo/a -> ../a)
            pagePath.append(last_pop)
            targetPath.append(last_pop)
    
        targetPath.reverse()
    
        relativeUrl = ['..'] * (len(pagePath))
        relativeUrl = ('', '', '/'.join(relativeUrl + targetPath)) + tuple(afterPath)
    
        return urlparse.urlunparse(relativeUrl)

def html2safehtml(s, valid_tags=('b', 'a', 'i', 'br', 'p'),pageurl=''):
    valid_tags = [t.lower() for t in valid_tags]
    parser = HTML2SafeHTML(valid_tags)
    if pageurl:
        # Submit the pageurl as base-url for calculating urls
        parser.pageurl = pageurl

    parser.feed(s)
    parser.close()
    parser.cleanup()
    return parser.result
