##############################################################################
#
# BernArticle - yet another Plone article...
#
# Copyright (C) 2005 4teamwork Ltd., Bern, Switzerland and
#                    Informatikdienste of the city of Bern, Switzerland
#
# GNU General Public Licence (GPL)
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307 USA
#
############################################################################## 

try:
    from Products.LinguaPlone.public import *
except ImportError: 
    # No multilingual support
    from Products.Archetypes.public import *

from AccessControl.SecurityManagement import *
from Products.CMFCore.tests.base.security import AnonymousUser
from Products.ATContentTypes.lib.constraintypes import ConstrainTypesMixinSchema
from Products.ATContentTypes.lib.constraintypes import ConstrainTypesMixin
from ZPublisher.HTTPRequest import HTTPRequest   
from AccessControl import ClassSecurityInfo
from Products.Archetypes.Marshall import RFC822Marshaller
from Products.Archetypes.public import MetadataSchema
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin


from Products.BernArticle.content.schemata.BernArticleSchema import BernArticle_schema



class BernArticle(OrderedBaseFolder, ConstrainTypesMixin, BrowserDefaultMixin):
    """A flexible folderish document type that can be composed of several individual content blocks."""

    __implements__ = (OrderedBaseFolder.__implements__,
                      ConstrainTypesMixin.__implements__,
                      BrowserDefaultMixin.__implements__) 

    
    security = ClassSecurityInfo()
    portal_type = meta_type = 'BernArticle' 
    archetype_name = 'BernArticle'   #this name appears in the 'add' box 

    filter_content_types = 1
    allowed_content_types = tuple() # late set by BernArticle registration process # ['BernArticle', 'BernArticleCollector']
 
    typeDescription= 'A flexible folderish document type that can be composed of several individual content blocks.'
    typeDescMsgId  = 'bernarticle_description_edit_BernArticle'
 
    _at_rename_after_creation = True # rename object according to the title?
    allow_discussion = 0
    global_allow = 1
    content_icon = 'bernarticle.gif'

    immediate_view = 'base_view'  # first view after addition
    default_view   = 'bernarticle_view'
    suppl_views    = ('bernarticle_view',)

    schema = BaseSchema.copy() + ConstrainTypesMixinSchema.copy() + BernArticle_schema


    actions=  (
        {
        'id': 'view',
        'name': 'View',
        'action': 'string:${object_url}',
        'permissions': (CMFCorePermissions.View,)
        },
        {
        'id': 'preview',
        'name': 'Preview',
        'condition': 'python: not hasattr(object.REQUEST,\'preview\')',
        'action': 'string:${object_url}?preview=1',
        'permissions': (CMFCorePermissions.ModifyPortalContent,)
        },
              { 'id': 'edit',
                 'name': 'Edit',
                 'action': 'string:${object_url}/base_edit',
                 'condition': 'python: not hasattr(object,\'isTranslatable\') or hasattr(object,\'isTranslatable\') and object.isTranslatable() and object.isCanonical()',
                 'permissions': (CMFCorePermissions.ModifyPortalContent,),
                 },
               { 'id': 'translate',
                 'name': 'Edit',
                 'action': 'string:${object_url}/translate_item',
                 'condition': 'python: hasattr(object,\'isTranslatable\') and object.isTranslatable() and not object.isCanonical()',
                 'permissions': (CMFCorePermissions.ModifyPortalContent,),
                 }, 
        {
        'id': 'local_roles',
        'name': 'Sharing',
        'action': 'string:${object_url}/folder_localrole_form',
        'permissions': (CMFCorePermissions.ManageProperties,),
        'category': 'object'
        },          
         
    )


    # initial value for article view:
    def _get_default_viewTemplate(self):
        try:
           return self.aq_parent.getViewTemplateName()
        except:
           return 'bernarticleblock_view_1'
    
    # initial value for listing of subarticles:    
    def _get_default_viewListing(self):
        return 'bernarticlelist_listing_3'
          
    def get_image_size_2column(self,width,space):
        return int((width-space)/2)
        
    def get_image_size_3column(self,width,space):
        return int((width-(2*space))/3)
        
    def get_image_size_4column(self,width,space):
        return int((width-(3*space))/4)

    def get_image_sizes(self):
        ptool = getToolByName(self,'portal_properties', None)
        if ptool and hasattr(ptool, 'BernArticle_properties'):
            try:
               BernArticle_width = int(ptool.BernArticle_properties.getProperty('BernArticle_width', None))
            except:
               BernArticle_width = 530
            try:
               BernArticle_image_space = int(ptool.BernArticle_properties.getProperty('BernArticle_image_space', None))
            except:
               BernArticle_image_space = 10
            try:
               BernArticle_size_thumbnail = int(ptool.BernArticle_properties.getProperty('BernArticle_size_thumbnail', None))
            except:
               BernArticle_size_thumbnail = 35
            try:
               BernArticle_fixed_image_height = int(ptool.BernArticle_properties.getProperty('BernArticle_fixed_image_height', None))
            except:
               BernArticle_fixed_image_height = 150
        else:
            BernArticle_width = 530
            BernArticle_image_space = 10
            BernArticle_size_thumbnail = 35
        size2 = self.get_image_size_2column(BernArticle_width,BernArticle_image_space)
        size3 = self.get_image_size_3column(BernArticle_width,BernArticle_image_space)
        size4 = self.get_image_size_4column(BernArticle_width,BernArticle_image_space)
        return {'thumbnail': (BernArticle_size_thumbnail, BernArticle_size_thumbnail*2), '1column': (BernArticle_width, (BernArticle_width*2)), '2column': (size2, size2*2), '3column': (size3, size3*2), '4column': (size4, size4*2), '0column': (BernArticle_width, BernArticle_fixed_image_height)}

    security.declareProtected(CMFCorePermissions.View, 'tag')
    def tag(self, **kwargs):
        """Generate image tag using the api of the ImageField
        """
        return self.getField('image1').tag(self, **kwargs)

    def exclude_from_nav(self):
        """Accessor for excludeFromNav field
        """
        field = self.getField('excludeFromNav')
        if field is not None:
            return field.get(self)
        else:
            return False

    def allowedContentTypes(self):
        return ConstrainTypesMixin.allowedContentTypes(self);

    def getBernArticle(self):
        """ returns the root BernArticle object (to be used through
            acquisition in order to be able to obtain the root document
            without having the need to climb up using aq_parent.
            (ajung)
        """
        return self


registerType(BernArticle)
