from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.migrations.migration_util import safeEditProperty
from Products.BernArticle.config import *
from StringIO import StringIO
from DateTime import DateTime

def installTextIndexNG(self, out):
    # Install TextIndexNG if present
    try:
        from Products.TextIndexNG2.TextIndexNG import TextIndexNG

        class args:
            def __init__(self, **kw):
                self.__dict__.update(kw)
            def keys(self):
                return self.__dict__.keys()

        ct = getToolByName(self, 'portal_catalog')
        extra = args(default_encoding='utf-8')

        indexes = {}
        for idx in ct.index_objects():
            indexes[idx.getId()] = idx

        if not isinstance(indexes.get('SearchableText',None), TextIndexNG):
            ct.delIndex('SearchableText')
            ct.addIndex('SearchableText', 'TextIndexNG2', 
                        extra=args(default_encoding='utf-8', 
                                   use_converters=1, autoexpand=1))

        if not isinstance(indexes.get('Title',None), TextIndexNG):
            ct.delIndex('Title')
            ct.addIndex('Title', 'TextIndexNG2', extra=extra)
            
        if not isinstance(indexes.get('Description',None), TextIndexNG):
            ct.delIndex('Description')
            ct.addIndex('Description', 'TextIndexNG2', extra=extra)
        
        ct.refreshCatalog(clear=1)

    except ImportError:
        pass

        
def installProducts(self, out):
    # install other products
    quickinstaller = getToolByName(self, 'portal_quickinstaller')
    for productname in PRODUCTLIST: 
        if not quickinstaller.isProductInstalled(productname):
            try:
                quickinstaller.installProduct(productname)
                # Refresh skins
                if hasattr(self, '_v_skindata'):
                    self._v_skindata = None
                if hasattr(self, 'setupCurrentSkin'):
                    self.setupCurrentSkin()
                out.write('Installed %s\n' % productname)
            except:
                out.write('Unable to install %s\n' % productname)
        else:
            out.write('%s already installed\n' % productname)
    
    if INSTALL_LINGUAPLONE == 0:
        try:
            if quickinstaller.isProductInstalled('LinguaPlone'):
                quickinstaller.uninstallProducts(['LinguaPlone'])
                #quickinstaller.reinstallProducts(['PloneLanguageTool'])
        except:
            pass
    

def setSiteProperties(self, out):
    # set general properties
    p=getattr(self.portal_properties, 'site_properties')
    safeEditProperty(p, 'default_charset', 'utf-8', 'string')
    safeEditProperty(p, 'localTimeFormat', '%d.%m.%Y', 'string')
    safeEditProperty(p, 'localLongTimeFormat', '%d.%m.%Y %H:%M', 'string')
    safeEditProperty(p, 'default_language', PLATFORM_DEFAULT_LANGUAGE, 'string')
    safeEditProperty(p, 'ext_editor', False, 'boolean')
    safeEditProperty(p, 'available_editors', DEFAULT_AVAILABLE_EDITORS,'lines')
    
    self.manage_changeProperties({'left_slots':LEFT_SLOTS})
    self.manage_changeProperties({'right_slots':RIGHT_SLOTS})
    self.manage_changeProperties({'email_from_address':DEFAULT_EMAIL_FROM_ADDRESS})
    self.manage_changeProperties({'email_from_name':DEFAULT_EMAIL_FROM_NAME})
    
    out.write('Properties edited.\n')


def add_workflow(self, wf_id, wf_factory_id, types):
    """ add a workflow definition to the portal_workflow tool """
    wf_tool = getToolByName(self, 'portal_workflow')

    try:
        wf_tool.manage_delObjects([wf_id])        
    except Exception, E:
        pass

    wf_tool.manage_addWorkflow( wf_factory_id, wf_id )    
    if types:
        wf_tool.setChainForPortalTypes(types, wf_id)    


def installWorkflows(self,out):
    import BernArticle_workflow
    reload(BernArticle_workflow)
    add_workflow(self, 'BernArticle_workflow', 'BernArticle_workflow (BernArticle Workflow)', ())
    wf_tool = getToolByName(self, 'portal_workflow')      
    wf_tool.setDefaultChain('BernArticle_workflow')
    
    
def installActions(self, out):
    actions_tool = getToolByName(self, 'portal_actions')

    #remove actions 
    actions=actions_tool._cloneActions()
    for action in actions:
        if action.id in ('news','Members','index_html'):
            action.visible = 0
    
    actions_tool._actions=tuple(actions)
    
    #add actions into portal_controlpanel
    try:
        portal_conf=getToolByName(self,'portal_controlpanel')
    except AttributeError:
        print >>out, "Configlet could not be installed"
        return
    try:
        portal_conf.registerConfiglet( 'PortalProperties'
               , 'Properties management'
               , 'string:${portal_url}/prefs_portal_properties_list'
               , ''                 # a condition
               , ('Manage portal',)    # access permission
               , 'Plone'         # section to which the configlet should be added: Plone,Products,Members
               , 1                  # visibility
               , PROJECTNAME
               , 'properties_icon.gif' # icon in control_panel
               , ''
               , None
               )
    except KeyError:
        pass # Get KeyError when registering duplicate configlet.

    # modify actions into the portal_controlpanel tool
    cp_tool = getToolByName(self, 'portal_controlpanel')
    cp_actions=cp_tool._cloneActions()
    for action in cp_actions:
        if action.id in ('PortalSkin',):
            action.visible = 0
        if action.id in ('QuickInstaller','ZMI','cmfmember','kupu'):
            action.permissions = ('View management screens',)
    cp_tool._actions=tuple(cp_actions)

    out.write("Actions added or modified.\n")

     
def prepopulateSite(self, out):
    # prepopulate the site with some first content objects and folders
    # will be done from each product's install method
    
    self.portal_catalog.refreshCatalog()

    # prepopulate
    for obj in PREPOPULATE:
        if obj['path'] == '':
            path = self
        else:
            path = getattr(self, obj['path'])
        o = getattr(path, obj['id'], None)
        if o:
            path.manage_delObjects(ids=[obj['id']])
        path.invokeFactory(type_name=obj['type'],
                           id=obj['id'],title=obj['title']
                           )
        o = getattr(path, obj['id'])
        
        if obj['types']:    
           o.setTypes(obj['types'])

        # try to change workflow state:
        try:
            self.portal_workflow.doActionFor(o, 'publish')
        except:
            pass

        o.reindexObject()
        out.write("Object reindexed.\n")


    # change the start page
    try:
        self.manage_delObjects(ids=['front-page'])   # removes the original plone front page
    except:
        pass
    self.invokeFactory(type_name='BernArticle', id='welcome', title='Welcome at a BernArticle Site')
    welcome = getattr(self, 'welcome')
    welcome.edit(description="This is a simple BernArticle page that serves as default view for the site. Feel free to edit as needed.",
                 text=WELCOME_TEXT)
    self.setDefaultPage('welcome')
    
    self.portal_catalog.refreshCatalog()
    out.write("Platform prepopulated.\n")
    

def modifyPortalTypes(self, out):
    # update the portal_types tool: make some type not implicitly addable
    pttool = getToolByName(self, 'portal_types')
    types = pttool.listContentTypes()
    for type in types: 
        if type in NOT_IMPLICITLY_ADDABLE:
            t = pttool.getTypeInfo(type)
            t.global_allow = 0
    out.write("Portal Types Tool updated.\n")

    
def modifyPortalCatalog(self, out):
    # update the portal_catalog tool: SearchableText as TextIndexNG2 and reindex it
    ct = getToolByName(self, 'portal_catalog')

#    for idx in ['SearchableText',]:
#        if idx in ct.indexes():
#           ct.manage_delIndex([idx])
#        ct.addIndex(idx, 'TextIndexNG2')
#        ct.manage_reindexIndex([idx])
#        out.write("Added index '%s' to the catalog.\n" % idx)

def modifyHttpCache(self, out):
    # update the HTTP Cache Manager Object
    httpcache = self.HTTPCache
    httpcache.manage_editProps(title='',settings={'anonymous_only':0,'interval':36000,'notify_urls':[]})
    out.write("HTTP Cache Manager updated.\n")
    
    
def modifyRolesAndPermissions(self, out):
    # add roles
    for role in DEFAULT_NEW_ROLES:
        if not role in self.valid_roles():
            self._addRole(role)
    
    # modify some permissions
    results = []
    perms = self.ac_inherited_permissions(1)
    perms = list(perms)
    for p in perms:
        perm = p[0]
        results.append(perm)
    self.manage_role('Administrator', permissions=results)
    self.manage_permission('View management screens',roles=['Manager',], acquire=0)
    self.manage_permission('Use mailhost services',roles=['Administrator','Anonymous','Authenticated','Manager','Member','Owner','Reviewer'], acquire=0)
    self.manage_permission('Add portal member',roles=['Manager','Administrator'], acquire=0)
    out.write("Roles modified.\n")

    
def modifyKupuTool(self, out):
    # update some properties of the kupu tool
    ktool = getToolByName(self, 'kupu_library_tool')
    ktool._res_types.update(KUPU_DEFAULT_RESOURCE_TYPES)
    out.write('Kupu Tool modified.\n')


def updateLanguageSettings(self, out):
    results = self.portal_catalog()
    for r in results:
        o = r.getObject()
        if o.language != PLATFORM_DEFAULT_LANGUAGE:
            oldmodi = DateTime(o.ModificationDate())
            o.setLanguage(PLATFORM_DEFAULT_LANGUAGE) #self.plone_utils.editMetadata(o,language='de')
            o.setModificationDate(oldmodi)

    # update portal_languages tool
    quickinstaller = getToolByName(self, 'portal_quickinstaller')
    if INSTALL_LINGUAPLONE == 0:
        try:
            quickinstaller.reinstallProducts(['PloneLanguageTool'])
        except:
            pass

    try:
        ltool = getToolByName(self, "portal_languages")
        ltool.setDefaultLanguage(PLATFORM_DEFAULT_LANGUAGE)
        ltool.addSupportedLanguage(PLATFORM_DEFAULT_LANGUAGE)
        ltool.removeSupportedLanguages(['en',])
    except:
        pass
    
    self.portal_memberdata.setLanguage(PLATFORM_DEFAULT_LANGUAGE)
    self.portal_memberdata.reindexObject()

    out.write('Languages properties updated.\n')
    
def setup(context):
    # main customization method
    out = StringIO()
    site = context.getSite()
    installTextIndexNG(site, out)
    installProducts(site, out)
    #installWorkflows(site,out)
    setSiteProperties(site,out)
    installActions(site, out)
    prepopulateSite(site, out)
    modifyPortalTypes(site, out)
    modifyRolesAndPermissions(site, out)
    modifyKupuTool(site, out)
    updateLanguageSettings(site, out)

